package example.shop.data;

import example.shop.Constants;

import javax.servlet.http.HttpSession;
import java.util.Vector;
import java.util.StringTokenizer;
import java.io.*;

public class DataManager {
    public static final String REL_DATA_FILE_NAME = "WEB-INF/data.dat";
    public static final String DATA_SEPARATOR = "\t";

    private String mPathToFile = null;
    private Vector mData = null;

    public static DataManager getFromSession(HttpSession aSession) {
        DataManager dm = (DataManager) aSession.getAttribute(Constants.SESSIONKEY_DATAMAN);

        if (dm == null) {
            System.out.println("DataMan is null");
        }
        return dm;
    }

    public DataManager(String aPathToFile) {
        mPathToFile = aPathToFile;
        mData = new Vector();
        loadData();
    }

    public void addItemInfo(ItemInfo aInfo) {
        mData.add(aInfo);
        saveData();
    }

    public void deleteItemInfo(String aName) {
        if (aName == null) return;

        for (int i = 0; i < mData.size(); i++) {
            ItemInfo info = (ItemInfo) mData.elementAt(i);
            if (info.getName().equals(aName)) {
                mData.remove(i);
                break;
            }
        }

        saveData();
    }

    public Vector getData() {
        return mData;
    }

    /**
     * Loads data from the file.<br>
     * Data is written in the following format<br>
     * <pre>
     *           <name>\t<price>\t<description>
     * </pre>
     */
    private void loadData() {

        if (mPathToFile == null) {
            return;
        }

        FileInputStream fis = null;
        InputStreamReader isr = null;
        BufferedReader br = null;
        System.out.println("Loading data....");
        try {
            fis = new FileInputStream(mPathToFile + REL_DATA_FILE_NAME);
            isr = new InputStreamReader(fis);
            br = new BufferedReader(isr);

            mData.clear();
            String line = null;
            while ( (line = br.readLine()) != null) {
                if (line.length() == 0) continue;

                System.out.println("   >>> " + line);
                StringTokenizer stok = new StringTokenizer(line, DATA_SEPARATOR);
                ItemInfo info = new ItemInfo();
                info.setName(stok.nextToken());
                info.setPrice(Double.valueOf(stok.nextToken()));
                info.setDescription(stok.nextToken());
                mData.add(info);
            }

        } catch (FileNotFoundException fnfe) {
            // do nothing - no file
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                if (br != null) br.close();
                if (isr != null) isr.close();
                if (fis != null) fis.close();
            } catch (IOException e) {
                System.out.println("Data file could not be closed during loading");
            }
        }

    }

    private void saveData() {
        FileWriter fw = null;
        PrintWriter pw = null;

        try {
            fw = new FileWriter(mPathToFile + REL_DATA_FILE_NAME, false); // do not append
            pw = new PrintWriter(fw);

            for (int i = 0; i < mData.size(); i++) {
                ItemInfo info = (ItemInfo) mData.elementAt(i);

                pw.print(info.getName());
                pw.print(DATA_SEPARATOR);
                pw.print(info.getPrice());
                pw.print(DATA_SEPARATOR);
                pw.print(info.getDescription());
                pw.println();
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                if (pw != null) pw.close();
                if (fw != null) fw.close();
            } catch (IOException e) {
                System.out.println("File could not be closed on saving");
            }
        }
    }
}
